#!/usr/bin/env python2
#
# GEemyle: Server View
# Copyright (C) 2003-2006 Easter-eggs <info@easter-eggs.com>
# Authors:
#      Cyril Lacoux (Yack) <clacoux@easter-eggs.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

__author__  = "Cyril Lacoux [Yack] <clacoux@easter-eggs.com>"
__cvsid__   = "$Id: server.py 102 2006-06-01 19:34:20Z yack $"
__version__ = "$Revision: 102 $"[11:-2]
__all__     = ["ServerNetwork"]


import pygtk
pygtk.require('2.0')
import gtk

import common
import geemyle.eemyle_connector as connector
import geemyle.gettext as gettext
import geemyle.gui.mygtk        as mygtk

_ = gettext._


class ServerDNS(common.ServerTable):

    def __init__(self, id_server, server=None, display_lists=None):

        common.ServerTable.__init__(self)

        self.id_server      = -1

        self.en_domain_name = gtk.Entry()

        self.construct_frame()
        if server:
            self.set_server(server)

    def construct_frame(self):

        i  = 0
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Domain name'))
        self.attach(label,               0, 1, i, i+1)
        self.attach(self.en_domain_name, 1, 2, i, i+1)

    def set_server(self, server):

        self.id_server = server['id']

        self.en_domain_name.set_text(server['domain_name'] or "")

    def get_server(self):

        server = { 'domain_name': self.en_domain_name.get_text() }

        return server

    def on_update_server_clicked(self, button):

        server = self.get_server()
        connector.eemyle_connector.send_request(
            'servers/%s/modify' % self.id_server,
            'update_server',
            method="POST",
            json=server
        )


class ServerInterface(common.ServerTable):

    def __init__(self, id_server, interface=None, display_lists=None):

        common.ServerTable.__init__(self)

        self.id_server    = id_server
        self.id_interface = -1

        self.en_name      = gtk.Entry()
        self.en_ip_addr   = gtk.Entry()
        self.en_netmask   = gtk.Entry()
        self.en_broadcast = gtk.Entry()
        self.en_gateway   = gtk.Entry()
        self.en_network   = gtk.Entry()
        self.cb_public    = gtk.CheckButton(_('Public'))

        self.construct_frame()
        if interface:
            self.set_interface(interface)

    def construct_frame(self):

        i  = 0
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Name'))
        self.attach(label,             0, 1, i, i+1)
        self.attach(self.en_name,      1, 2, i, i+1)

        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('IP address'))
        self.attach(label,             0, 1, i, i+1)
        self.attach(self.en_ip_addr,   1, 2, i, i+1)
        self.attach(self.cb_public,    2, 3, i, i+1)

        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Network'))
        self.attach(label,             0, 1, i, i+1)
        self.attach(self.en_network,   1, 2, i, i+1)

        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Netmask'))
        self.attach(label,             2, 3, i, i+1)
        self.attach(self.en_netmask,   3, 4, i, i+1)

        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Gateway'))
        self.attach(label,             0, 1, i, i+1)
        self.attach(self.en_gateway,   1, 2, i, i+1)

        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Broadcast'))
        self.attach(label,             2, 3, i, i+1)
        self.attach(self.en_broadcast, 3, 4, i, i+1)

    def set_interface(self, interface):

        self.id_interface = interface['id']

        self.en_name.set_text(interface['name'] or "")
        self.en_ip_addr.set_text(interface['ip'] or "")
        self.en_netmask.set_text(interface['netmask'] or "")
        self.en_broadcast.set_text(interface['broadcast'] or "")
        self.en_network.set_text(interface['network'] or "")
        self.en_gateway.set_text(interface['gateway'] or "")
        if interface['pub']:
            self.cb_public.set_active(True)

    def get_interface(self):

        interface = { 'name':         self.en_name.get_text(),
                      'ip':           self.en_ip_addr.get_text(),
                      'netmask':      self.en_netmask.get_text(),
                      'broadcast':    self.en_broadcast.get_text(),
                      'network':      self.en_network.get_text(),
                      'gateway':      self.en_gateway.get_text(),
                      'pub':          self.cb_public.get_active() }


        return interface

    def on_add_interface_activate(self):

        dialog = common.AddDialog(self, _('add an interface'))

        refresh = False
        loop = True
        while loop:
            response = dialog.run()
            if response == gtk.RESPONSE_OK:
                interface = self.get_interface()

                result, data = connector.eemyle_connector.send_direct_request(
                    "servers/%s/interfaces/add" % self.id_server,
                    method="POST",
                    json=interface
                )
                if result and data:
                    loop = False
                    refresh = True
            else:
                loop = False

        dialog.destroy()

        if refresh:
            connector.eemyle_connector.send_request(
                "servers/%s" % self.id_server,
                "get_server_by_server_id",
                json={"details": ["users", "usages", "interfaces", "other_customer_hosts"]}
            )

    def on_remove_interface_clicked(self, button):

        dialog = common.RemoveDialog(message_format=_('Do you really want to delete interface %s from this server ?') % self.en_name.get_text())

        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            connector.eemyle_connector.send_request(
                "servers/%s/interfaces/%s/delete" % (self.id_server, self.id_interface),
                "remove_interface",
                method="DELETE"
            )

        dialog.destroy()

    def on_update_interface_clicked(self, button):

        interface = self.get_interface()
        connector.eemyle_connector.send_request(
            "servers/%s/interfaces/%s/modify" % (self.id_server, self.id_interface),
            "update_interface",
            method="POST",
            json=interface
        )

class ServerNetwork(common.ServerTab):

    def __init__(self):

        common.ServerTab.__init__(self, label=_('Network'), has_add=True, add_label=_('Add _interface'))

        self.id_server = -1

        # Buttons:
        self.bt_add.connect('clicked', self.on_add_interface_clicked)

    def set_server(self, server=None):

        self.remove_children()
        self.set_sensitive(False)

        if not server:
            self.id_server = -1
            return True

        self.id_server = server['id']
        self.set_sensitive(True)

        # Dns
        dns   = ServerDNS(self.id_server, server=server)

        frame = common.HardwareFrame(_('DNS'), has_remove=False)
        frame.bt_save.connect('clicked', dns.on_update_server_clicked)
        frame.set_sensitive(True)
        frame.pack_start(dns, expand=False)

        self.add_child(frame, expand=False)

        # Network interfaces:
        if server['interfaces']:
            i = 1
            for id_interface, interface in server['interfaces'].items():
                iface = ServerInterface(self.id_server, interface=interface)

                frame = common.HardwareFrame(_('Interface #%d') % i)
                frame.bt_remove.connect('clicked', iface.on_remove_interface_clicked)
                frame.bt_save.connect('clicked',   iface.on_update_interface_clicked)
                frame.set_sensitive(True)
                frame.pack_start(iface, expand=False)

                self.add_child(frame, expand=False)
                i += 1

        self.show_all()

    def on_add_interface_clicked(self, button):

        if self.id_server == -1:
            return False

        iface = ServerInterface(self.id_server)
        iface.on_add_interface_activate()

